@@ -65,7 +65,6 @@ import warnings
 import datetime as dt
 
 import numpy as np
-import matplotlib.mlab
 
 from spacepy import help
 #Try to pull in the C version. Assumption is that if you import this module,
@@ -217,7 +216,6 @@ class PPro(object):
                 first_idx = [bisect.bisect_left(p2, starts[nss] + self.lags[ilag])
                              for nss in nss_list]
                 self.n_assoc[ilag, :] = [last_idx[i] - first_idx[i] for i in nss_list]
-            pul = matplotlib.mlab.prctile_rank(lags, p=(20,80))
         else:
             def fracday(dd):
                 '''turn a timedelta into a fractional day'''
@@ -244,10 +242,11 @@ class PPro(object):
             n_assoc = self.n_assoc.ctypes.data_as(lib.lptr)
             lib.assoc(p2, p1, lags.ctypes.data_as(lib.dptr), n_assoc,
                       winhalf, len(p2), len(p1), len(self.lags))
-            pul = matplotlib.mlab.prctile_rank(lags, p=(20,80))
+        left20perc = int(np.round((len(lags)*0.2)))
+        right20perc = int(np.round((len(lags)*0.8)))
         self.assoc_total = np.sum(self.n_assoc, axis=1)
-        valsL = self.assoc_total[pul==0]
-        valsR = self.assoc_total[pul==2]
+        valsL = self.assoc_total[:left20perc]
+        valsR = self.assoc_total[right20perc:]
         self.asympt_assoc = np.mean([np.mean(valsL), np.mean(valsR)])
 
         self.expected = np.empty([len(self.lags)], dtype='float64')
@@ -482,29 +481,29 @@ class PPro(object):
 
         Warnings
         ========
-        If ``seed`` is specified on numpy 1.5 and earlier, the available
-        entropy is reduced to work around a random number generator bug.
-        Upgrade to numpy 1.6 to avoid this limitation.
-        Because of this workaround, if a seed is specified, results
-        from numpy 1.5 are not reproducible with numpy 1.6
+        If ``seed`` is specified, results may not be reproducible between
+        systems with different sizes for C long type. Note that 64-bit
+        Windows uses a 32-bit long and so results will be the same between
+        64 and 32-bit Windows, but not between 64-bit Windows and other 64-bit
+        operating systems. If ``seed`` is not specified, results are
+        not reproducible anyhow.
         """
         lags = self.lags
 
         ci_low = np.empty([len(lags)])
         ci_high = np.empty([len(lags)])
         conf_above = np.empty([len(lags)])
+        long_size = ctypes.sizeof(ctypes.c_long) * 8
 
         if seed != None:
             np.random.seed(seed)
-            #TODO: This is a bit ugly and we potentially lose entropy
-            #should be unsigned long, but numpy forces signed int...
-            minseed = -sys.maxsize - 1
-            maxseed = sys.maxsize
-            (maj, minor) = np.__version__.split('.')[0:2]
-            if int(maj) < 2 and int(minor) < 6:
-                warnings.warn('Upgrade to numpy 1.6 to avoid reduced entropy.')
-                maxseed = sys.maxsize
-                minseed = 0
+            minseed = -2 ** (long_size - 1)
+            maxseed = 2 ** (long_size - 1) - 1
+            #randint used to be system-size signed integer only.
+            #so used that and cast to the required unsigned later
+            #cast does not lose entropy: negative numbers map to high positives.
+            #For reproducibility, keep doing that even though dtype
+            #kwarg now available.
             lag_seeds = np.random.randint(minseed, maxseed, [len(lags)])
             newtype = np.dtype('u' + str(lag_seeds.dtype))
             lag_seeds = np.require(lag_seeds, dtype=newtype)
@@ -519,7 +518,7 @@ class PPro(object):
         else:
             perc_low = (100.-inter)/2. #set confidence interval
             perc_high = inter + perc_low
-            dtype = 'int' + str(ctypes.sizeof(ctypes.c_long) * 8)
+            dtype = 'int' + str(long_size)
             assoc_totals = np.empty([len(lags), n_boots],
                                     dtype=dtype, order='C')
             if seed == None:
@@ -537,8 +536,8 @@ class PPro(object):
             tb.thread_job(len(lags), 0, thread_targ)
             for i in range(len(lags)):
                 assoc_totals[i, :].sort()
-                ci_low[i], ci_high[i] = matplotlib.mlab.prctile(
-                    assoc_totals[i, :], p=(perc_low,perc_high))
+                ci_low[i], ci_high[i] = np.percentile(
+                    assoc_totals[i, :], (perc_low,perc_high))
                 conf_above[i] = 100.0 - value_percentile(assoc_totals[i, :],
                                                          self.asympt_assoc)
         self.ci = [ci_low, ci_high]
@@ -757,16 +756,17 @@ def boots_ci(data, n, inter, func, seed=None, target=None, sample_size=None, use
                   ctypes.c_ulong(n), ctypes.c_ulong(n_els),
                   ctypes.c_ulong(sample_size),
                   ctypes.c_ulong(seed), clock_seed)
-        surr_quan = sorted(
-            (func(surr_ser[i * sample_size:(i  + 1) * sample_size])
-             for i in range(n)))
+        gen = (func(surr_ser[i * sample_size:(i  + 1) * sample_size])
+               for i in range(n))
+        #Can't sort if more than one return value
+        surr_quan = sorted(gen) if nretvals == 1 else np.stack(gen)
     #get confidence interval
     if nretvals>1:
         pul = np.empty((2, nretvals))
         for nn in range(nretvals):
-            pul[:,nn] = matplotlib.mlab.prctile(surr_quan[:,nn], p=(perc_low,perc_high))
+            pul[:,nn] = np.percentile(surr_quan[:,nn], (perc_low,perc_high))
     else:
-        pul = matplotlib.mlab.prctile(surr_quan, p=(perc_low,perc_high))
+        pul = np.percentile(surr_quan, (perc_low,perc_high))
     if target == None:
         return pul[0], pul[1]
     else:
