@@ -177,7 +177,8 @@ def parse_filename_time(filename):
 
     # Look for run time:
     if '_t' in filename:
-        runtime = float(re.search('_t(\d+)', filename).groups()[-1])
+        raw     = re.search('_t(\d+)', filename).groups()[-1]
+        runtime = 3600*float(raw[:-4]) + 60*float(raw[-4:-2]) + float(raw[-2:])
     else:
         runtime = None
 
@@ -247,7 +248,8 @@ def parse_tecvars(line):
     return ret
 
 
-def add_planet(ax, rad=1.0, ang=0.0, add_night=True, **extra_kwargs):
+def add_planet(ax, rad=1.0, ang=0.0, add_night=True, zorder=1000,
+               **extra_kwargs):
     '''
     Creates a circle of ``radius=self.para['rbody']`` and returns the
     MatPlotLib Ellipse patch object for plotting.  If an axis is specified
@@ -276,22 +278,26 @@ def add_planet(ax, rad=1.0, ang=0.0, add_night=True, **extra_kwargs):
        Defaults to zero (terminator is aligned with Y-axis.)
     add_night : boolean
        Add night hemisphere.  Defaults to **True**
+    zorder : int
+       Set the matplotlib zorder of the patch to set how other plot
+       elements order with the inner boundary patch. Defaults to 1000,
+       nightside patch is given zorder of *zorder+5*.
     
     '''
 
     from matplotlib.patches import Circle, Wedge
 
-    body = Circle((0,0), rad, fc='w', zorder=1000, **extra_kwargs)
+    body = Circle((0,0), rad, fc='w', zorder=zorder, **extra_kwargs)
     arch = Wedge((0,0), rad, 90+ang, -90+ang, fc='k', 
-                 zorder=1001, **extra_kwargs)
+                 zorder=zorder+5, **extra_kwargs)
         
     ax.add_artist(body)
     if add_night: ax.add_artist(arch)
 
     return body, arch
 
-def add_body(ax, rad=2.5, facecolor='lightgrey', add_planet=True, 
-             ang=0.0, add_night=True, **extra_kwargs):
+def add_body(ax, rad=2.5, facecolor='lightgrey', show_planet=True, 
+             ang=0.0, add_night=True, zorder=1000, **extra_kwargs):
     '''
     Creates a circle of radius=self.attrs['rbody'] and returns the
     MatPlotLib Ellipse patch object for plotting.  If an axis is specified
@@ -301,7 +307,7 @@ def add_body(ax, rad=2.5, facecolor='lightgrey', add_planet=True,
     
     Because the body is rarely the size of the planet at the center of 
     the modeling domain, add_planet is automatically called.  This can
-    be negated by using the add_planet kwarg.
+    be negated by using the show_planet kwarg.
     
     Parameters
     ==========
@@ -315,7 +321,7 @@ def add_body(ax, rad=2.5, facecolor='lightgrey', add_planet=True,
     facecolor : string
        Set color of face of inner boundary circle via Matplotlib color 
        selectors (name, hex, etc.)  Defaults to 'lightgrey'.
-    add_planet : boolean
+    show_planet : boolean
        Turns on/off planet indicator inside inner boundary.  
        Defaults to **True**
     ang : float
@@ -323,16 +329,20 @@ def add_body(ax, rad=2.5, facecolor='lightgrey', add_planet=True,
        Defaults to zero (terminator is aligned with Y-axis.)
     add_night : boolean
        Add night hemisphere.  Defaults to **True**
+    zorder : int
+       Set the matplotlib zorder of the patch to set how other plot
+       elements order with the inner boundary patch. Defaults to 1000.
+       If a planet is added, it is given a zorder of *zorder*+5.
 
     '''
     from matplotlib.patches import Ellipse
     
     dbody = 2.0 * rad
-    body = Ellipse((0,0),dbody,dbody,facecolor=facecolor, zorder=999,
+    body = Ellipse((0,0),dbody,dbody,facecolor=facecolor, zorder=zorder,
                    **extra_kwargs)
 
-    if DoPlanet:
-        add_planet(ax, ang=ang, add_night=add_night)
+    if show_planet:
+        add_planet(ax, ang=ang, add_night=add_night, zorder=zorder+5)
 
     ax.add_artist(body)
         
@@ -471,6 +481,7 @@ def _read_idl_ascii(pbdat, header='units', keep_case=True):
     gridnames = names[:ndim]
     if gtyp == 'Irregular':
         for v in names:
+            if v not in pbdat: continue
             pbdat[v] = dmarray(np.reshape(pbdat[v], pbdat['grid'], order='F'),
                                attrs=pbdat[v].attrs)
     elif gtyp == 'Regular':
@@ -480,11 +491,83 @@ def _read_idl_ascii(pbdat, header='units', keep_case=True):
             pbdat[x] = dmarray(pbdat[x][0:prod[i+1]-prod[i]+1:prod[i]],
                 attrs=pbdat[x].attrs)
         for v in names:
+            if v not in pbdat.keys(): continue
             if v not in pbdat['grid'].attrs['dims']:
                 pbdat[v] = dmarray(np.reshape(pbdat[v], pbdat['grid'],
                                               order='F'), attrs=pbdat[v].attrs)
 
-def _read_idl_bin(pbdat, header='units', keep_case=True):
+def readarray(f,dtype=np.float32,inttype=np.int32):
+    """
+    Read an array from an unformatted binary file written out by a 
+    Fortran program.
+
+    :param f: File to read
+    :type f: File-like
+    :param nrec: Number of records to read. If nrec is greater than one, 
+    all records read will be appended together into a single 1-D array.
+    :type nrec: int
+    """
+
+    if dtype is str:
+        dtype_size_bytes=1
+    else:
+        dtype_size_bytes=dtype.itemsize
+
+    # Get the record length
+    rec_len=np.fromfile(f,dtype=inttype,count=1)
+
+    # Check that the record length is consistent with the data type
+    if rec_len%dtype_size_bytes!=0:
+        raise ValueError('Read error: Data type inconsistent with record' +
+                         ' length (data type size is {0:d} bytes, record length is {1:d} bytes'.format(int(dtype_size_bytes),int(n)))
+
+    if len(rec_len)==0:
+        # Zero-length record...file may be truncated
+        raise EOFError('Zero-length read at start marker')
+
+    try:
+        startpos = f.tell()
+    except IOError:
+        seekable = False
+    else:
+        seekable = True
+    if seekable:
+        f.seek(0, 2)
+        endpos = f.tell()
+        if endpos - startpos < (int(rec_len[0]) + np.dtype(inttype).itemsize):
+            raise EOFError('File is shorter than expected data')
+        f.seek(startpos + int(rec_len[0]), 0)
+        rec_len_end = np.fromfile(f,dtype=inttype,count=1)
+        f.seek(startpos, 0)
+        if rec_len_end != rec_len:
+            raise ValueError((
+                'Read error: End marker length ({0:d}) does not match start '
+                'marker length ({1:d}).').format(rec_len[0], rec_len_end[0]) + 
+                'This indicates incorrect endiannes, wrong file type, '
+                'or file is corrupt.')
+    # Read the data
+    if dtype is str:
+        A=f.read(rec_len[0])
+    else:
+        A=np.fromfile(f,dtype=dtype,count=int(rec_len[0]/dtype_size_bytes))
+
+    # Check the record length marker at the end
+    rec_len_end=np.fromfile(f,dtype=inttype,count=1)
+    if len(rec_len_end)==0:
+        # Couldn't read, file may be truncated
+        raise EOFError('Zero-length read at end marker')
+
+    if rec_len_end!=rec_len:
+        # End marker is inconsistent with start marker. Something is wrong.
+        raise ValueError(
+            'Read error: End marker does not match start marker '+
+            '(start marker says record length is {0:d} bytes, end marker says {1:d} bytes).'.format(int(rec_len),int(rec_len_end)) +
+            'This indicates incorrect endiannes, wrong file type, '+
+            'or file is corrupt'.format(int(rec_len),int(rec_len_end)))
+
+    return A
+
+def _read_idl_bin(pbdat, header='units', keep_case=True, headeronly=False):
     '''
     Load a SWMF IDL binary output file and load into a pre-existing PbData
     object.  This should only be called by :class:`IdlFile`.
@@ -523,155 +606,165 @@ def _read_idl_bin(pbdat, header='units', keep_case=True):
     # Open, read, and parse the file into numpy arrays.
     # Note that Fortran writes integer buffers around records, so
     # we must parse those as well.
-    infile = open(pbdat.attrs['file'], 'rb')
-    
-    # On the first try, we may fail because of wrong-endianess.
-    # If that is the case, swap that endian and try again.
-    EndChar = '<' # Endian marker (default: little.)
-    pbdat.attrs['endian']='little'
-    RecLenRaw = infile.read(4)
-    
-    RecLen = ( struct.unpack(EndChar+'l', RecLenRaw) )[0]
-    if (RecLen > 10000) or (RecLen < 0):
-        EndChar = '>'
-        pbdat.attrs['endian']='big'
-        RecLen = ( struct.unpack(EndChar+'l', RecLenRaw) )[0]
+    with open(pbdat.attrs['file'], 'rb') as infile:
+        # On the first try, we may fail because of wrong-endianess.
+        # If that is the case, swap that endian and try again.
+        endian='little'
+
+        inttype=np.dtype(np.int32)
+        EndChar='<'
+        inttype.newbyteorder(EndChar)
+
+        try:
+            headline=readarray(infile,str,np.int32)
+        except (ValueError,EOFError):
+            endian='big'
+            EndChar='>'
+            inttype.newbyteorder(EndChar)
+            infile.seek(0)
+            headline=readarray(infile,str,)
+        headline=headline.decode('utf-8')
         
-    headline = ( struct.unpack('{0}{1}s'.format(EndChar, RecLen),
-                             infile.read(RecLen)) )[0].strip()
-    if str is not bytes:
-        headline = headline.decode()
+        pbdat.attrs['endian']=endian
 
-    (OldLen, RecLen) = struct.unpack(EndChar+'2l', infile.read(8))
-    pformat = 'f'
-    # parse rest of header; detect double-precision file.
-    if RecLen > 20: pformat = 'd'
-    (pbdat.attrs['iter'], pbdat.attrs['runtime'],
-     pbdat.attrs['ndim'], pbdat.attrs['nparam'], pbdat.attrs['nvar']) = \
-        struct.unpack('{0}l{1}3l'.format(EndChar, pformat), infile.read(RecLen))
-    # Get gridsize
-    (OldLen, RecLen) = struct.unpack(EndChar+'2l', infile.read(8))
-    #pbdat['grid']=dmarray(struct.unpack(EndChar+'%il' % abs(pbdat.attrs['ndim']),  
-    pbdat['grid']=dmarray(struct.unpack('{0}{1}l'.format(EndChar, abs(pbdat.attrs['ndim'])), 
-                                       infile.read(RecLen)))
-    # Data from generalized (structured but irregular) grids can be 
-    # detected by a negative ndim value.  Unstructured grids (e.g.
-    # BATS, AMRVAC) are signified by negative ndim values AND
-    # the grid size is always [x, 1(, 1)]
-    # Here, we set the grid type attribute to either Regular, 
-    # Generalized, or Unstructured.  Let's set that here.
-    pbdat['grid'].attrs['gtype'] = 'Regular'
-    pbdat['grid'].attrs['npoints']  = abs(pbdat['grid'].prod())
-    if pbdat.attrs['ndim'] < 0: 
-        if any(pbdat['grid'][1:] > 1): 
-            pbdat['grid'].attrs['gtype'] = 'Generalized'
+        # detect double-precision file.
+        pos=infile.tell()
+
+        RecLen=np.fromfile(infile,dtype=inttype,count=1)
+        infile.seek(pos)
+
+        # Set data types
+        if RecLen > 20:
+            floattype=np.dtype(np.float64)
         else:
-            pbdat['grid'].attrs['gtype']   = 'Unstructured'
-            pbdat['grid'].attrs['npoints'] = pbdat['grid'][0]
-    pbdat.attrs['ndim'] = abs(pbdat.attrs['ndim'])
+            floattype=np.dtype(np.float32)
+        floattype.newbyteorder(EndChar)
 
-    # Quick ref vars:
-    time=pbdat.attrs['runtime']
-    gtyp=pbdat['grid'].attrs['gtype']
-    npts=pbdat['grid'].attrs['npoints']
-    ndim=pbdat['grid'].size
-    nvar=pbdat.attrs['nvar']
-    npar=pbdat.attrs['nparam']
+        # Parse rest of header
+        header_fields_dtype=np.dtype([
+            ('it',np.int32),('t',floattype),('ndim',np.int32),
+            ('npar',np.int32),('nvar',np.int32)])
+        header_fields_dtype.newbyteorder(EndChar)
 
-    # Read parameters stored in file.
-    para  = np.zeros(npar)
-    if npar>0:
-        (OldLen, RecLen) = struct.unpack(EndChar+'2l', infile.read(8))
-        para[:] = struct.unpack('{0}{1}{2}'.format(EndChar, npar, pformat),
-                                infile.read(RecLen))
+        (pbdat.attrs['iter'], pbdat.attrs['runtime'],
+         pbdat.attrs['ndim'], pbdat.attrs['nparam'], pbdat.attrs['nvar']) = \
+            readarray(infile,
+                      dtype=header_fields_dtype,
+                      inttype=inttype)[0]
 
-    (OldLen, RecLen) = struct.unpack(EndChar+'2l', infile.read(8))
-    names = ( struct.unpack('{0}{1}s'.format(EndChar, RecLen),
-                            infile.read(RecLen)) )[0]
-    if str is not bytes:
-        names = names.decode()
+        # Get gridsize
+        pbdat['grid']=dmarray(readarray(infile,inttype,inttype))
 
-    # Preserve or destroy original case of variable names:
-    if not keep_case: names = names.lower()
+        # Data from generalized (structured but irregular) grids can be
+        # detected by a negative ndim value.  Unstructured grids (e.g.
+        # BATS, AMRVAC) are signified by negative ndim values AND
+        # the grid size is always [x, 1(, 1)]
+        # Here, we set the grid type attribute to either Regular,
+        # Generalized, or Unstructured.  Let's set that here.
+        pbdat['grid'].attrs['gtype'] = 'Regular'
+        pbdat['grid'].attrs['npoints']  = abs(pbdat['grid'].prod())
+        if pbdat.attrs['ndim'] < 0:
+            if any(pbdat['grid'][1:] > 1):
+                pbdat['grid'].attrs['gtype'] = 'Generalized'
+            else:
+                pbdat['grid'].attrs['gtype']   = 'Unstructured'
+                pbdat['grid'].attrs['npoints'] = pbdat['grid'][0]
+        pbdat.attrs['ndim'] = abs(pbdat.attrs['ndim'])
 
-    names.strip()
-    names = names.split()
+        # Quick ref vars:
+        time=pbdat.attrs['runtime']
+        gtyp=pbdat['grid'].attrs['gtype']
+        npts=pbdat['grid'].attrs['npoints']
+        ndim=pbdat['grid'].size
+        nvar=pbdat.attrs['nvar']
+        npar=pbdat.attrs['nparam']
 
-    # Now that we know the number of variables, we can properly handle
-    # the headline and units based on the kwarg *header*:
-    pbdat.attrs['header']=headline
-    if header == 'units':
-        # If headline is just units:
-        units = headline.split()
-    else:
-        # If headline is NOT just units, create blank units:
-        units = [''] * (len(names)-npar)
-    
-    # For some reason, there are often more units than variables
-    # in these files.  It looks as if there are more grid units
-    # than grid vectors (e.g. 'R R R' implies X, Y, and Z data
-    # in file but only X and Y are present.)  Let's try to work
-    # around this rather egregious error.
-    nSkip = len(units)+npar-len(names)
-    if nSkip<0: nSkip = 0
-    
-    # Save grid names (e.g. 'x' or 'r') and save associated params.
-    pbdat['grid'].attrs['dims']=names[0:ndim]
-    for name, para in zip(names[(nvar+ndim):], para):
-        pbdat.attrs[name]=para
-            
-            
-    # Create string representation of time.
-    pbdat.attrs['strtime'] = '{0:04d}h{1:02d}m{2:06.3f}s'.format(int(time//3600), int(time%3600//60), time%60)
+        # Read parameters stored in file.
+        para  = np.zeros(npar)
+        if npar>0:
+            para[:] = readarray(infile,floattype,inttype)
 
-    # Get the grid points...
-    (OldLen, RecLen) = struct.unpack(EndChar+'2l', infile.read(8))
-    prod = [1] + pbdat['grid'].cumprod().tolist()
-    for i in range(0,ndim):
-        # Read the data into a temporary grid.
-        tempgrid = np.array(struct.unpack('{0}{1}{2}'.format(EndChar, npts, pformat),
-                                          infile.read(int(RecLen//ndim)) ) )
-        # Unstructred grids get loaded as vectors.
+        names = readarray(infile,str,inttype).decode('utf-8')
+
+        # Preserve or destroy original case of variable names:
+        if not keep_case: names = names.lower()
+
+        names.strip()
+        names = names.split()
+
+        # Now that we know the number of variables, we can properly handle
+        # the headline and units based on the kwarg *header*:
+        pbdat.attrs['header']=headline
+        if header == 'units':
+            # If headline is just units:
+            units = headline.split()
+        else:
+            # If headline is NOT just units, create blank units:
+            units = [''] * (len(names)-npar)
+        
+        # For some reason, there are often more units than variables
+        # in these files.  It looks as if there are more grid units
+        # than grid vectors (e.g. 'R R R' implies X, Y, and Z data
+        # in file but only X and Y are present.)  Let's try to work
+        # around this rather egregious error.
+        nSkip = len(units)+npar-len(names)
+        if nSkip<0: nSkip = 0
+        
+        # Save grid names (e.g. 'x' or 'r') and save associated params.
+        pbdat['grid'].attrs['dims']=names[0:ndim]
+        for name, para in zip(names[(nvar+ndim):], para):
+            pbdat.attrs[name]=para
+                
+                
+        # Create string representation of time.
+        pbdat.attrs['strtime'] = '{0:04d}h{1:02d}m{2:06.3f}s'.format(
+            int(time//3600), int(time%3600//60), time%60)
+
+        # Get the grid points...
+        prod = [1] + pbdat['grid'].cumprod().tolist()
+
+        # Read the data into a temporary array
+        griddata = readarray(infile,floattype,inttype)
+        for i in range(0,ndim):
+            # Get the grid coordinates for this dimension
+            tempgrid = griddata[npts*i:npts*(i+1)]
+
+            # Unstructred grids get loaded as vectors.
+            if gtyp == 'Unstructured':
+                pbdat[names[i]] = dmarray(tempgrid)
+            # Irregularly gridded items need multidimensional grid arrays:
+            elif gtyp == 'Irregular':
+                pbdat[names[i]] = dmarray(np.reshape(tempgrid, pbdat['grid'],
+                                                     order='F'))
+            # Regularly gridded ones need vector grid arrays:
+            elif gtyp == 'Regular':
+                pbdat[names[i]] = dmarray(np.zeros(pbdat['grid'][i]))
+                for j in range(int(pbdat['grid'][i])):
+                    pbdat[names[i]][j] = tempgrid[j*int(prod[i])]
+            else:
+                raise ValueError('Unknown grid type: {0}'.format(pbdat.gridtype))
+            # Add units to grid.
+            if units: pbdat[names[i]].attrs['units'] = units.pop(nSkip)
+
+        # Get the actual data and sort.
+        for i in range(ndim,nvar+ndim):
+            pbdat[names[i]] = dmarray(readarray(infile,floattype,inttype))
+            if units: pbdat[names[i]].attrs['units']=units.pop(nSkip)
+            if gtyp != 'Unstructured':
+                # Put data into multidimensional arrays.
+                pbdat[names[i]] = pbdat[names[i]].reshape(pbdat['grid'], order='F')
+
+        # Unstructured data can be in any order, so let's sort it.
         if gtyp == 'Unstructured':
-            pbdat[names[i]] = dmarray(tempgrid)
-        # Irregularly gridded items need multidimensional grid arrays:
-        elif gtyp == 'Irregular':
-            pbdat[names[i]] = dmarray(np.reshape(tempgrid, pbdat['grid'],
-                                                 order='F'))
-        # Regularly gridded ones need vector grid arrays:
-        elif gtyp == 'Regular':
-            pbdat[names[i]] = dmarray(np.zeros(pbdat['grid'][i]))
-            for j in range(int(pbdat['grid'][i])):
-                pbdat[names[i]][j] = tempgrid[j*int(prod[i])]
-        else:
-            raise ValueError('Unknown grid type: {0}'.format(pbdat.gridtype))
-        # Add units to grid.
-        pbdat[names[i]].attrs['units'] = units.pop(nSkip)
-
-    # Get the actual data and sort.
-    for i in range(ndim,nvar+ndim):
-        (OldLen, RecLen) = struct.unpack(EndChar+'2l', infile.read(8))
-        pbdat[names[i]] = dmarray(
-            np.array(struct.unpack('{0}{1}{2}'.format(EndChar, npts, pformat),
-                                       infile.read(RecLen))) )
-        pbdat[names[i]].attrs['units'] = units.pop(nSkip)
-        if gtyp != 'Unstructured':
-            # Put data into multidimensional arrays.
-            pbdat[names[i]] = pbdat[names[i]].reshape(pbdat['grid'], order='F')
-
-    # Unstructured data can be in any order, so let's sort it.
-    if gtyp == 'Unstructured':
-        gridtotal = np.zeros(npts)
-        offset = 0.0  # The offset ensures no repeating vals while sorting.
-        for key in pbdat['grid'].attrs['dims']:
-            gridtotal = gridtotal + offset + pbdat[key]
-            offset = offset + np.pi/2.0
-            SortIndex = np.argsort(gridtotal)
-        for key in list(pbdat.keys()):
-            if key=='grid': continue
-            pbdat[key] = pbdat[key][SortIndex]
-
-    infile.close()
+            gridtotal = np.zeros(npts)
+            offset = 0.0  # The offset ensures no repeating vals while sorting.
+            for key in pbdat['grid'].attrs['dims']:
+                gridtotal = gridtotal + offset + pbdat[key]
+                offset = offset + np.pi/2.0
+                SortIndex = np.argsort(gridtotal)
+            for key in list(pbdat.keys()):
+                if key=='grid': continue
+                pbdat[key] = pbdat[key][SortIndex]
 
 class PbData(SpaceData):
     '''
@@ -781,7 +874,7 @@ class IdlFile(PbData):
     as an "unpack requires a string of argument length 'X'".
     '''
 
-    def __init__(self, filename,format='binary',header='units',
+    def __init__(self, filename,format=None,header='units',
                  keep_case=True, *args,**kwargs):
         super(IdlFile, self).__init__(*args, **kwargs)  # Init as PbData.
         self.attrs['file']   = filename   # Save file name.
@@ -798,8 +891,13 @@ class IdlFile(PbData):
         set when the object is instantiation.
         '''
 
-        if self.attrs['format'][:3] == 'bin':
-            _read_idl_bin(  self, header=header, keep_case=keep_case)
+        if self.attrs['format'] is None:
+            try:
+                _read_idl_bin(self, header=header, keep_case=keep_case)
+            except (ValueError, EOFError, MemoryError):
+                _read_idl_ascii(self, header=header, keep_case=keep_case)
+        elif self.attrs['format'][:3] == 'bin':
+            _read_idl_bin(self, header=header, keep_case=keep_case)
         elif self.attrs['format'][:3] == 'asc':
             _read_idl_ascii(self, header=header, keep_case=keep_case)
         else:
@@ -908,15 +1006,9 @@ class LogFile(PbData):
         self.attrs['npts']=npts
 
         # Pop time/date/iteration names off of Namevar.
-        if 'year'in loc: names.pop(names.index('year'))
-        if 'mo'  in loc: names.pop(names.index('mo'))
-        if 'dy'  in loc: names.pop(names.index('dy'))
-        if 'hr'  in loc: names.pop(names.index('hr'))
-        if 'mn'  in loc: names.pop(names.index('mn'))
-        if 'sc'  in loc: names.pop(names.index('sc'))
-        if 'msc' in loc: names.pop(names.index('msc'))
-        if 't'   in loc: names.pop(names.index('t'))
-        if 'it'  in loc: names.pop(names.index('it'))
+        for key in ['year','mo','dy','hr','mn','sc','msc',
+                    't','it','yy','mm','dd','hh','ss','ms']:
+            if key in loc: names.pop(names.index(key))
 
         # Create containers for data:
         time=dmarray(np.zeros(npts, dtype=object))
@@ -928,17 +1020,30 @@ class LogFile(PbData):
         for i in range(npts):
             vals = raw[i].split()
             # Set time:
-            if 'year' in loc:
-                # If "year" is listed, we have the full datetime.
-                time[i]=(dt.datetime(
-                        int(vals[loc['year']]), # Year
-                        int(vals[loc['mo']  ]), # Month
-                        int(vals[loc['dy']]), # Day
-                        int(vals[loc['hr']]), # Hour
-                        int(vals[loc['mn']]), # Minute
-                        int(vals[loc['sc']]), # Second
-                        int(vals[loc['msc']]) * 1000 #microsec
-                        ))
+            if 'year' in loc or 'yy' in loc:
+                # If "year" or "yy" is listed, we have the full datetime.
+                if 'year' in loc:
+                    # BATS date format
+                    time[i]=(dt.datetime(
+                            int(vals[loc['year']]), # Year
+                            int(vals[loc['mo']  ]), # Month
+                            int(vals[loc['dy']]), # Day
+                            int(vals[loc['hr']]), # Hour
+                            int(vals[loc['mn']]), # Minute
+                            int(vals[loc['sc']]), # Second
+                            int(vals[loc['msc']]) * 1000 #microsec
+                            ))
+                elif 'yy' in loc:
+                    # RIM date format
+                    time[i]=(dt.datetime(
+                            int(vals[1]), # Year
+                            int(vals[2]), # Month
+                            int(vals[3]), # Day
+                            int(vals[4]), # Hour
+                            int(vals[5]), # Minute
+                            int(vals[6]), # Second
+                            int(vals[7]) * 1000 #microsec
+                            ))
                 diffT = time[i] - time[0]
                 runtime[i]=diffT.days*24.0*3600.0 + \
                     diffT.seconds + \
@@ -1424,6 +1529,8 @@ class ImfInput(PbData):
             out.write(head)
 
         # Handle Params:
+        if self.attrs['coor']:
+            out.write('#COOR\n{}\n\n'.format(self.attrs['coor']))
         if self.attrs['zerobx']:
             out.write('#ZEROBX\nT\n\n')
         if self.attrs['reread']:
@@ -1453,6 +1560,83 @@ class ImfInput(PbData):
                 ' '.join('{:10.2f}'.format(self[key][i]) for key in var)))
         out.close()
 
+    def add_pram_bz(self, target=None, loc=111, pcol='#CC3300', bcol='#3333CC',
+                    xlim=None, plim=None, blim=None, epoch=None):
+        '''
+        Plot, on a single set of axes, both ram pressure and IMF Bz for a 
+        quick-look summary of an event. 
+
+        The figure and main axes object are returned.
+
+        Parameters
+        ==========
+
+        Other Parameters
+        ================
+        target : Figure or Axes
+            If None (default), a new figure is generated from scratch.
+            If a matplotlib Figure object, a new axis is created
+            to fill that figure.
+            If a matplotlib Axes object, the plot is placed
+            into that axis.
+        loc : int
+            Use to specify the subplot placement of the axis
+            (e.g. loc=212, etc.) Used if target is a Figure or None.
+            Default 111 (single plot).
+        pcol : matplotlib color specifier
+            Set the line and label color for ram pressure.
+        bcol : matplotlib color specifier
+            Set the line and label color for IMF Bz.
+        xlim : 2-element list of datetimes
+            Set the time range for the plot.  Defaults to full range.
+        plim : 2-element list
+            Set the y-axis range for dynamic pressure.
+        blim : 2-element list
+            Set the y-axis range for IMF Bz.
+        epoch : datetime.datetime
+            Create an epoch marker (vertical line) at time=epoch.
+
+        Returns
+        =======
+        fig : matplotlib figure object
+        ax  : matplotlib axes object
+        '''
+
+        import matplotlib.pyplot as plt
+        from spacepy.plot import set_target, applySmartTimeTicks
+        
+        # Set ax and fig based on given target.
+        fig, a1 = set_target(target, figsize=(8,4), loc=loc)
+        a2 = a1.twinx()
+        
+        self.calc_pram()
+
+        # Plot lines:
+        a1.plot(self['time'], self['bz'],   color=bcol)
+        a2.plot(self['time'], self['pram'], color=pcol, lw=1.5)
+
+        # Restrict x-range:
+        if not xlim: xlim=self['time']
+        applySmartTimeTicks(a1, xlim, dolabel=True)
+
+        # Zero line for IMF Bz:
+        a1.hlines(0, xlim[0], xlim[-1], color=bcol, linestyles='dashed')
+
+        # Label axes:
+        a1.set_ylabel('IMF B$_{Z}$ ($nT$)', color=bcol, size=16)
+        a2.set_ylabel('P$_{dyn}$ ($nPa$)',  color=pcol, size=16)
+        plt.setp(a1.get_yticklabels(), color=bcol)
+        plt.setp(a2.get_yticklabels(), color=pcol)
+
+        # Add epoch marker.
+        if epoch:
+            ymin, ymax = a1.get_ylim()
+            a1.vlines(epoch, ymin, ymax, linestyles='solid', color='g', 
+                      linewidths=2.)
+            a1.set_ylim([ymin, ymax])
+
+        return fig, a1
+        
     def quicklook(self, timerange=None):
         '''
         Generate a quick-look plot of solar wind conditions driving the
@@ -1504,7 +1688,7 @@ class ImfInput(PbData):
 
         a5 = fig.add_subplot(515)
         a5.plot(self['time'], -1.0*self['ux'], lw=1.25, c='green')
-        adjust_plots(a5, '$V_{X}$ ($km/s$)', Zero=False, xlab=True)
+        adjust_plots(a5, '$|V_{X}|$ ($km/s$)', Zero=False, xlab=True)
 
         return fig
 
