@@ -62,7 +62,7 @@ except NameError:
 
 __all__ = ['tOverlap', 'tOverlapHalf', 'tCommon', 'loadpickle', 'savepickle', 'assemble',
            'human_sort', 'feq', 'dictree', 'update', 'progressbar',
-           'windowMean', 'medAbsDev', 'binHisto',
+           'windowMean', 'medAbsDev', 'binHisto', 'bootHisto',
            'logspace', 'geomspace', 'linspace', 'arraybin', 'mlt2rad',
            'rad2mlt', 'pmm', 'getNamedPath', 'query_yes_no',
            'interpol', 'normalize', 'intsolve', 'dist_to_list',
@@ -570,7 +570,7 @@ def human_sort( l ):
     ['r1.txt', 'r2.txt', 'r10.txt']
     """
     convert = lambda text: int(text) if text.isdigit() else text
-    alphanum_key = lambda key: [ convert(c) for c in re.split('([0-9]+)', key) ]
+    alphanum_key = lambda key: [ convert(c) for c in re.split(r'([0-9]+)', key) ]
     alphanum_key = None
     try:
         l.sort( key=alphanum_key )
@@ -934,10 +934,10 @@ def indsFromXrange(inxrange):
     if not isinstance(inxrange, xrange): return None
     valstr = inxrange.__str__()
     if ',' not in valstr:
-        res = re.search('(\d+)', valstr)
+        res = re.search(r'(\d+)', valstr)
         retval = [int(0), int(res.group(1))]
     else:
-        res = re.search('(\d+), (\d+)', valstr)
+        res = re.search(r'(\d+), (\d+)', valstr)
         retval = [int(res.group(1)), int(res.group(2))]
     return retval
 
@@ -1187,23 +1187,22 @@ def binHisto(data, verbose=False):
     >>> data = numpy.random.randn(1000)
     >>> binw, nbins = spacepy.toolbox.binHisto(data)
     >>> print(nbins)
-    19.0
-    >>> p = plt.hist(data, bins=nbins, histtype='step', normed=True)
+    19
+    >>> p = plt.hist(data, bins=nbins, histtype='step', density=True)
 
     See Also
     ========
     matplotlib.pyplot.hist
     """
-    from matplotlib.mlab import prctile
-    pul = prctile(data, p=(25,75)) #get confidence interval
+    pul = np.percentile(data, (25, 75)) #get confidence interval
     ql, qu = pul[0], pul[1]
     iqr = qu-ql
     binw = 2.*iqr/(len(data)**(1./3.))
     if binw != 0:
-        nbins = round((max(data)-min(data))/binw)
+        nbins = int(round((max(data)-min(data))/binw))
     # if nbins is 0, NaN or inf don't use the F-D rule just use sqrt(num) rule
-    if binw == 0 or nbins == 0 or not np.isfinite(nbins):
-        nbins = round(np.sqrt(len(data)))
+    if binw == 0 or nbins == 0 or not np.isfinite(nbins) or nbins >= len(data)/2.:
+        nbins = int(round(np.sqrt(len(data))))
         binw = len(data)/nbins
         if verbose:
             print("Used sqrt rule")
@@ -1212,6 +1211,107 @@ def binHisto(data, verbose=False):
             print("Used F-D rule")
     return (binw, nbins)
 
+def bootHisto(data, inter=90., n=1000, seed=None,
+              plot=False, target=None, figsize=None, loc=None, **kwargs):
+    """Bootstrap confidence intervals for a histogram.
+
+    All other keyword arguments are passed to :func:`numpy.histogram`
+    or :func:`matplotlib.pyplot.bar`.
+
+    Parameters
+    ==========
+
+    data : array_like
+        list/array of data values
+    inter : float (optional; default 90)
+        percentage confidence interval to return. Default 90% (i.e.
+        lower CI will be 5% and upper will be 95%)
+    n : int (optional; default 100)
+        number of bootstrap iterations
+    seed : int (optional)
+        Optional seed for the random number generator. If not
+        specified; numpy generator will not be reseeded.
+    plot : bool (optional)
+        Plot the result. Plots if True or ``target``, ``figsize``,
+        or ``loc`` specified.
+    target : (optional)
+        Target on which to plot the figure (figure or axes). See
+        :func:`spacepy.plot.utils.set_target` for details.
+    figsize : tuple (optional)
+        Passed to :func:`spacepy.plot.utils.set_target`.
+    loc : int (optional)
+        Passed to :func:`spacepy.plot.utils.set_target`.
+
+    Returns
+    =======
+    out : tuple
+      tuple of bin_edges, low, high, sample[, bars]. Where
+      ``bin_edges`` is the edges of the bins used; ``low`` is the
+      histogram with the value for each bin from the bottom of that
+      bin's confidence interval; ``high`` similarly for the top;
+      ``sample`` is the histogram of the input sample without
+      resampling. If plotting, also returned is ``bars``, the
+      container object returned from matplotlib.
+
+    Notes
+    =====
+    The confidence intervals are calculated for each bin individually and thus
+    the resulting low/high histograms may not have actually occurred in the
+    calculation from the surrogates. If using a probability density histogram,
+    this can have "interesting" implications for interpretation.
+
+    Examples
+    ========
+    >>> import numpy.random
+    >>> import spacepy.toolbox
+    >>> numpy.random.seed(0)
+    >>> data = numpy.random.randn(1000)
+    >>> bin_edges, ci_low, ci_high, sample, bars = spacepy.toolbox.bootHisto(
+            data, plot=True)
+
+    .. plot::
+
+       import numpy.random
+       import spacepy.toolbox
+       numpy.random.seed(0)
+       data = numpy.random.randn(1000)
+       bin_edges, ci_low, ci_high, sample, bars = spacepy.toolbox.bootHisto(
+            data, plot=True)
+
+    See Also
+    ========
+    binHisto
+    plot.utils.set_target
+    numpy.histogram
+    matplotlib.pyplot.hist
+    """
+    import spacepy.poppy
+    histogram_allowed_kwargs = (
+        'bins', 'range', 'normed', 'weights', 'density')
+    histogram_kwargs = {k: v for k, v in kwargs
+                        if k in histogram_allowed_kwargs}
+    bar_kwargs = {k: v for k, v in kwargs
+                  if k not in histogram_allowed_kwargs}
+    sample, bin_edges = np.histogram(data, **histogram_kwargs)
+    histogram_kwargs['bins'] = bin_edges
+    ci_low, ci_high = spacepy.poppy.boots_ci(
+        data, n, inter,
+        lambda x: np.histogram(x, **histogram_kwargs)[0],
+        nretvals=len(bin_edges) - 1)
+    if not plot and all([x is None for x in (target, figsize, loc)]):
+        return bin_edges, ci_low, ci_high, sample
+    import spacepy.plot.utils
+    _, ax = spacepy.plot.utils.set_target(
+        target, figsize=figsize, loc=(111 if loc is None else loc))
+    if 'ecolor' not in bar_kwargs:
+        bar_kwargs['ecolor'] = 'k'
+    bars = ax.bar(
+        bin_edges[:-1], height=sample, width=np.diff(bin_edges),
+        align='edge', yerr=np.stack((sample - ci_low, ci_high - sample)),
+        **bar_kwargs)
+    return bin_edges, ci_low, ci_high, sample, bars
+
+
 def logspace(min, max, num, **kwargs):
     """
     Returns log-spaced bins. Same as numpy.logspace except the min and max are the min and max
@@ -1932,14 +2032,18 @@ def quaternionConjugate(Qin, scalarPos='last'):
 
 # -----------------------------------------------
 
-def normalize(vec):
+def normalize(vec, low=0.0, high=1.0):
     """
-    Given an input vector normalize the vector
+    Given an input vector normalize the vector to a given range
 
     Parameters
     ==========
     vec : array_like
         input vector to normalize
+    low : float
+        minimum value to scale to, default 0.0
+    high : float
+        maximum value to scale to, default 1.0
 
     Returns
     =======
@@ -1952,14 +2056,8 @@ def normalize(vec):
     >>> tb.normalize([1,2,3])
     [0.0, 0.5, 1.0]
     """
-    # check to see if vec is numpy array, this is fastest
-    if isinstance(vec, np.ndarray):
-        out = (vec - vec.min())/np.ptp(vec)
-    else:
-        vecmin = np.min(vec)
-        ptp = np.ptp(vec)
-        out = [(val -  vecmin)/ptp for val in vec]
-    return out
+    return np.interp(vec, (np.nanmin(vec), np.nanmax(vec)), (low, high))
+
 
 def intsolve(func, value, start=None, stop=None, maxit=1000):
     """
@@ -2502,7 +2600,7 @@ def do_with_timeout(timeout, target, *args, **kwargs):
     t = ReturningThread(None, target, None, args, kwargs)
     t.start()
     retval = t.join(timeout)
-    if t.isAlive():
+    if t.is_alive():
         raise TimeoutError()
     return retval
 
@@ -2560,6 +2658,58 @@ def timeout_check_call(timeout, *args, **kwargs):
     return 0
 
 
+def poisson_fit(data, initial=None, method='Powell'):
+    """
+    Fit a Poisson distribution to data using the method and initial guess provided.
+
+    Parameters
+    ==========
+    data : array-like
+        Data to fit a Poisson distribution to.
+    initial : int or None
+        initial guess for the fit, if None np.median(data) is used
+    method : str
+        method passed to scipy.optimize.minimize, default='Powell'
+
+    Examples
+    ========
+    >>> import spacepy.toolbox as tb
+    >>> from scipy.stats import poisson
+    >>> import matplotlib.pyplot as plt
+    >>> import numpy as np
+    >>> data = poisson.rvs(20, size=1000)
+    >>> res = tb.poisson_fit(data)
+    >>> print(res.x)
+    19.718000038769095
+    >>> xvals = np.arange(0, np.max(data)+5)
+    >>> plt.hist(data, bins=xvals, normed=True)
+    >>> plt.plot(xvals, poisson.pmf(xvals, np.round(res.x)))
+
+    Returns
+    =======
+    result : scipy.optimize.optimize.OptimizeResult
+        Resulting fit results from scipy.optimize, answer is result.x,
+        user should likely round.
+
+    """
+    from scipy.optimize import minimize
+    from scipy.stats import poisson
+
+    def negLogLikelihood(params, data):
+        """ the negative log-Likelihood-Function"""
+        return (- np.sum(np.log(poisson.pmf(data, params[0]))))
+
+    if initial is None:
+        initial = np.median(data)
+
+    ans = minimize(negLogLikelihood,  # function to minimize
+                   x0=initial,  # start value
+                   args=(data,),  # additional arguments for function
+                   method=method,  # minimization method, see docs
+                   )
+    return ans
+
+
 if __name__ == "__main__":
     import doctest
     doctest.testmod()
