@@ -11,6 +11,7 @@ import time, datetime
 import glob, os, sys
 import shutil
 import random
+import re
 import tempfile
 try:
     import StringIO
@@ -27,10 +28,18 @@ except ImportError:
 import numpy
 from numpy import array
 from scipy import inf
+from scipy.stats import poisson
+
 import spacepy.toolbox as tb
 import spacepy.time as st
 import spacepy.lib
 
+#Py3k compatibility renamings
+try:
+    xrange
+except NameError:
+    xrange = range
+
 __all__ = ['PickleAssembleTests', 'SimpleFunctionTests', 'TBTimeFunctionTests',
            'ArrayBinTests']
 
@@ -104,8 +113,19 @@ class PickleAssembleTests(unittest.TestCase):
 
 
 class SimpleFunctionTests(unittest.TestCase):
+    def test_quaternionNormalize(self):
+        """quaternionNormalize should have known results"""
+        tst = tb.quaternionNormalize([0.707, 0, 0.707, 0.2])
+        ans = [ 0.69337122,  0.        ,  0.69337122,  0.19614462]
+        numpy.testing.assert_array_almost_equal(ans, tst)
+
+    def test_indsFromXrange(self):
+        """indsFromXrange should have known result"""
+        foo = xrange(23, 39)
+        self.assertEqual([23, 39], tb.indsFromXrange(foo))
+
     def test_interweave(self):
-        """interweave should hav known result"""
+        """interweave should have known result"""
         a = numpy.arange(5)
         b = numpy.arange(5, 10)
         numpy.testing.assert_equal(numpy.vstack((a,b)).reshape((-1,),order='F'),
@@ -115,7 +135,7 @@ class SimpleFunctionTests(unittest.TestCase):
 
     def test_getNamedPath(self):
         """getNamedPath should have known result"""
-        curloc = os.path.abspath(os.path.curdir)
+        curloc = os.path.dirname(os.path.abspath(__file__))
         tmpdir = os.path.join(curloc, 'tmp', 'test1', 'test2')
         os.makedirs(tmpdir)
         ans = ['tests', 'tmp', 'test1']
@@ -239,9 +259,23 @@ class SimpleFunctionTests(unittest.TestCase):
         numpy.testing.assert_equal(real_ans, tb.interpol([1.5, 10.5, 359.5], x, y, wrap=360.0))
 
     def test_normalize(self):
-        """normalize should give known results"""
-        self.assertEqual([0.0, 0.5, 1.0], tb.normalize([1,2,3]))
-        numpy.testing.assert_equal(array([0.0, 0.5, 1.0]), tb.normalize(array([1,2,3])))
+        """normalize should give known results, default range"""
+        numpy.testing.assert_array_almost_equal(array([0.0, 0.5, 1.0]), tb.normalize(array([1,2,3])))
+
+    def test_normalize_small(self):
+        """normalize should give known results, smaller range"""
+        numpy.testing.assert_array_almost_equal(array([0.1 , 0.45, 0.8 ]), tb.normalize(array([1,2,3]), low=0.1, high=0.8))
+
+    def test_normalize_large(self):
+        """normalize should give known results, larger range"""
+        numpy.testing.assert_array_almost_equal(array([1.0, 6.0, 11.0]), tb.normalize(array([1,2,3]), low=1, high=11))
+
+    def test_normalize_nan(self):
+        """normalize should give known results, larger range with nan"""
+        numpy.testing.assert_array_almost_equal(array([1.0, 6.0, 11.0,numpy.nan]), tb.normalize(array([1,2,3,numpy.nan]),
+                                                                                   low=1, high=11))
+        numpy.testing.assert_array_almost_equal(array([1.0, 6.0, numpy.nan, 11.0]), tb.normalize(array([1,2,numpy.nan, 3,]),
+                                                                                    low=1, high=11))
 
     def testfeq_equal(self):
         """feq should return true when they are equal"""
@@ -268,6 +302,13 @@ class SimpleFunctionTests(unittest.TestCase):
         ans = tb.medAbsDev(data)
         self.assertAlmostEqual(ans, real_ans, places=1)
 
+    def test_medAbsDev_scale(self):
+        """medAbsDev should return a known range for given random input"""
+        data = numpy.random.normal(0, 1, 100000)
+        real_ans = 0.7*1.4826
+        ans = tb.medAbsDev(data, scale=True)
+        self.assertAlmostEqual(ans, real_ans, places=1)
+
     def test_binHisto(self):
         """binHisto should return know answer for known input"""
         input = list(range(0, 101))
@@ -293,6 +334,28 @@ class SimpleFunctionTests(unittest.TestCase):
         self.assertEqual(result, "Used F-D rule\n")
         sys.stdout = realstdout
 
+    def testBootHisto(self):
+        """Bootstrap histogram known output for known input"""
+        numpy.random.seed(28420)
+        data = numpy.random.randn(1000)
+        bin_edges, ci_low, ci_high, sample = spacepy.toolbox.bootHisto(
+            data, n=1000, seed=28420)
+        numpy.testing.assert_allclose(
+            [-3.17371804, -2.40693385, -1.64014966, -0.87336547,
+             -0.10658127, 0.66020292,  1.42698711,  2.1937713 ,
+             2.9605555 ,  3.72733969, 4.49412388],
+            bin_edges, rtol=1e-5)
+        numpy.testing.assert_equal(
+            [  7,  45, 149, 283, 272, 162,  71,   9,   0,   2], sample)
+        #This is a very coarse chunk-check to allow for variations in
+        #RNGs. Values from 100000 iterations
+        numpy.testing.assert_allclose(
+            [3.,  34., 131., 260., 249., 143.,  58.,   4.,   0.,   0.],
+            ci_low, atol=2, rtol=1e-2)
+        numpy.testing.assert_allclose(
+            [12.,  56., 168., 306., 295., 181.,  84.,  14.,   0.,   5.],
+            ci_high, atol=2, rtol=1e-2)
+
     def test_logspace(self):
         """logspace should return know answer for known input"""
         try:
@@ -530,6 +593,7 @@ class SimpleFunctionTests(unittest.TestCase):
         b = a[0:10]
         self.assertTrue(tb.isview(b))
         self.assertFalse(tb.isview(a))
+        self.assertFalse(tb.isview([1, 2, 3]))
         numpy.testing.assert_array_equal(tb.isview(a, [1,2,3]), [False, False]) # a bit of a pathological case
         numpy.testing.assert_array_equal(tb.isview(b, a), [True, True])
         numpy.testing.assert_array_equal(tb.isview(b, a[2:]), [True, False])
@@ -581,7 +645,15 @@ class SimpleFunctionTests(unittest.TestCase):
                             [1, 1, 1, 1, 1]])
         numpy.testing.assert_array_equal(ans1, tb.unique_columns(a, axis=1))
         numpy.testing.assert_array_equal(ans0, tb.unique_columns(a, axis=0))
-        
+
+    def test_poisson_fit(self):
+        """Make sure that we get the right Poisson fit answer"""
+        numpy.random.seed(8675309)
+        ans = 20
+        data = poisson.rvs(ans, size=1000)
+        res = tb.poisson_fit(data)
+        self.assertEqual(ans, numpy.round(res.x))
+
             
 class TBTimeFunctionTests(unittest.TestCase):
     def setUp(self):
@@ -737,9 +809,12 @@ class TBTimeFunctionTests(unittest.TestCase):
         sys.stdout = realstdout
         result = output.getvalue()
         output.close()
-#        numpy.testing.assert_allclose(t2-t1, 0.25, atol=0.1, rtol=0.1)
-        numpy.testing.assert_almost_equal(t2-t1, 0.25, decimal=1)
-        self.assertTrue(result in ("""('0.25', '')\n""", """('0.26', '')\n"""))
+        #There may be some overhead that pushes this past 0.25, but should
+        #never be less
+        self.assertTrue(0.25 <= t2 - t1 < 0.28)
+        m = re.match(r"^\('(\d\.\d\d)', ''\)\n$", result)
+        self.assertTrue(m)
+        self.assertTrue(0.25 <= float(m.group(1)) < 0.28)
 
     def test_windowMean_outputTimes(self):
         '''windowMean should return a known set of output times for a given set of input times and windows'''
@@ -937,6 +1012,17 @@ class TBTimeFunctionTests(unittest.TestCase):
         time = list(range(len(time)))
         self.assertRaises(TypeError, tb.windowMean, data, time, overlap=olap, st_time=datetime.datetime(2001,1,1))
 
+    def test_windowMean_2d(self):
+        """windowMean flattens and extends the time var over all the other dims"""
+        wsize = 2
+        olap = 0
+        data = numpy.arange(20).reshape(10,2)
+        out = tb.windowMean(data, winsize=wsize, overlap=olap)
+        ansd = [  1.5,   5.5,   9.5,  13.5]
+        numpy.testing.assert_almost_equal(ansd, out[0])
+        anst = [ 1.,  3.,  5.,  7.]
+        numpy.testing.assert_almost_equal(anst, out[1])
+
 
 class ArrayBinTests(unittest.TestCase):
     """Tests for arraybin function"""
